@@ -28,9 +28,9 @@
 /* XXX: copy between two remote sites */
 
 #include "includes.h"
-RCSID("$OpenBSD: sftp-client.c,v 1.33 2002/06/23 09:30:14 deraadt Exp $");
+RCSID("$OpenBSD: sftp-client.c,v 1.35 2002/09/11 22:41:49 djm Exp $");
 
-#include "openbsd-compat/fake-queue.h"
+#include "openbsd-compat/sys-queue.h"
 
 #include "buffer.h"
 #include "bufaux.h"
@@ -414,12 +414,6 @@ do_lsreaddir(struct sftp_conn *conn, char *path, int printflag,
 	return(0);
 }
 
-int
-do_ls(struct sftp_conn *conn, char *path)
-{
-	return(do_lsreaddir(conn, path, 1, NULL));
-}
-
 int
 do_readdir(struct sftp_conn *conn, char *path, SFTP_DIRENT ***dir)
 {
@@ -1095,7 +1089,7 @@ do_upload(struct sftp_conn *conn, char *local_path, char *remote_path,
 			debug3("In write loop, ack for %u %u bytes at %llu",
 			   ack->id, ack->len, (unsigned long long)ack->offset);
 			++ackid;
-			free(ack);
+			xfree(ack);
 		}
 		offset += len;
 	}
